require('lang/Object')

---@class Cursor : Object
---@field private _player LuaPlayer
Cursor = extends(Object)

---@return Cursor
---@param player_index number
function Cursor.load(player_index)
    local this = Cursor:super()
    this._player = game.players[player_index]
    return this
end

---@public
---@return boolean
function Cursor:holdsItem()
    return self:holdsSomething() and not self:holdsTool()
end

---@public
---@return boolean
function Cursor:holdsTool()
    return self:holdsSomething() and self:itemType() == "selection-tool"
end

---@private
---@return boolean
function Cursor:holdsSomething()
    return not self._player.is_cursor_empty() and (self:hasStack() or self:hasGhost())
end

---@private
---@return string
function Cursor:itemType()
    return self:item().type
end

---@public
---@return string
function Cursor:itemName()
    return self:item().name
end

---@private
---@return LuaItemStack | LuaItemPrototype | string
function Cursor:item()
    if self:hasStack() then
        return self._player.cursor_stack
    elseif self:hasGhost() then
        return self._player.cursor_ghost
    else
        error("Called for an item when cursor is empty")
    end
end

---@public
---@return boolean
function Cursor:hasStack()
    return self._player.cursor_stack and self._player.cursor_stack.valid_for_read and self._player.cursor_stack.name
end

---@public
---@return LuaItemStack
function Cursor:stack()
    return self._player.cursor_stack
end

---@private
---@return boolean
function Cursor:hasGhost()
    return self._player.cursor_ghost and self._player.cursor_ghost.name
end

---@public
---@param stack LuaItemStack
---@return boolean successful
function Cursor:grabStack(stack)
    self:clear()
    return self._player.cursor_stack.transfer_stack(stack)
end

---@public
---@param itemName string
function Cursor:grabGhost(itemName)
    self:clear()
    self._player.cursor_ghost = itemName
end

---@public
---@param toolName string
function Cursor:setTool(toolName)
    self:clear()
    self._player.cursor_stack.set_stack { name = toolName, count = 1 }
end

---@public
function Cursor:clear()
    self._player.clear_cursor()
end
