require('lang/Object')

---@class Header : Object
---@field _element LuaGuiElement
Header = extends(Object)
Header.name = "header"

---@public
---@param element LuaGuiElement
---@return boolean
function Header.is(element)
    return element.type == "flow" and element.name == Header.name
end

---@param parent LuaGuiElement
---@return Header
function Header.create(parent)
    local header = parent.add { type = "flow", name = Header.name, direction = "horizontal", style = "toolbar_header" }
    header.drag_target = parent
    header.add { type = "sprite-button", name = "command_remove_column", sprite = "interface-mod_left", style = "toolbar_command_button" }
    header.add { type = "sprite-button", name = "command_remove_row", sprite = "interface-mod_up", style = "toolbar_command_button" }
    header.add { type = "sprite-button", name = "command_add_row", sprite = "interface-mod_down", style = "toolbar_command_button" }
    header.add { type = "sprite-button", name = "command_add_column", sprite = "interface-mod_right", style = "toolbar_command_button" }
    header.add { type = "empty-widget", name = "space", style = "toolbar_header_space" }.drag_target = parent
    header.add { type = "sprite-button", name = "command_destroy_toolbar", sprite = "utility/trash", style = "toolbar_command_button_red" }
    return Header.load(header)
end

---@public
---@param element LuaGuiElement
---@return Header
function Header.load(element)
    local this = Header:super()
    this._element = Object.requireNonNil(element)
    return this
end

---@public
---@param guiEvent GuiEvent
function Header:handle(guiEvent)
    local element = guiEvent:element()
    if Button.is(element) then
        self:execute(Button.load(element):name())
    end
end

---@private
---@param command string
function Header:execute(command)
    if command == "command_destroy_toolbar" then
        self:toolbar():destroy()
    elseif command == "command_add_row" then
        self:toolbar():addRow()
    elseif command == "command_remove_row" then
        self:toolbar():removeRow()
    elseif command == "command_add_column" then
        self:toolbar():addColumn()
    elseif command == "command_remove_column" then
        self:toolbar():removeColumn()
    end
end

---@private
---@return Toolbar
function Header:toolbar()
    return Toolbar.load(self._element.parent)
end
