require('lang/Object')
require('model/slot/Slot')

---@class Table : Object
---@field private _element LuaGuiElement
Table = extends(Object)

---@param parent LuaGuiElement
---@return Table
function Table.create(parent)
    local panel = parent.add { type = "frame", name = "inner_panel", direction = "vertical", style = "toolbar_inner_panel" }
    local table = panel.add { type = "flow", name = "table", direction = "vertical", style = "toolbar_table" }
    return Table.load(table)
end

---@return Table
---@param element LuaGuiElement
function Table.load(element)
    local this = Table:super()
    this._element = Object.requireNonNil(element)
    return this
end

function Table:update()
    for _, item in pairs(self:items()) do
        item:updateCount()
    end
end

---@private
---@return Item[]
function Table:items()
    local items = {}
    for _, row in pairs(self:rows()) do
        for _, element in pairs(row.children) do
            if Item:is(element) then
                table.insert(items, Item.load(element))
            end
        end
    end
    return items
end

function Table:addRow()
    local row = self._element.add { type = "flow", name = "row_" .. self:rowsCount() + 1, direction = "horizontal", style = "toolbar_table_row" }
    for _ = 1, self:columnsCount() ~= 0 and self:columnsCount() or 5 do
        Slot.create(row)
    end
end

function Table:removeLastRow()
    if self:rowsCount() > 0 then
        self._element.children[self:rowsCount()].destroy()
    end
end

function Table:addColumn()
    for _, row in ipairs(self:rows()) do
        Slot.create(row)
    end
end

function Table:removeLastColumn()
    if self:columnsCount() > 0 then
        local columnsCount = self:columnsCount()
        for _, row in ipairs(self:rows()) do
            row.children[columnsCount].destroy()
        end
    end
end

---@private
---@return number
function Table:rowsCount()
    return #self:rows()
end

---@private
---@return LuaGuiElement[]
function Table:rows()
    return self._element.children
end

---@private
---@return number
function Table:columnsCount()
    if self:rowsCount() > 0 then
        return #self:firstRow().children
    else
        return 0
    end
end

---@private
---@return LuaGuiElement
function Table:firstRow()
    return self._element.children[1]
end
