require('lang/Object')
require('model/Table')
require('model/GuiEvent')
require('model/Button')
require('model/Header')

---@class Toolbar : Object
---@field _element LuaGuiElement
Toolbar = extends(Object)
Toolbar.pattern = "^toolbar_(%d+)$"

---@public
---@param element LuaGuiElement
function Toolbar.is(element)
    return element.type == "frame" and element.name:match(Toolbar.pattern)
end

---@param parent LuaGuiElement
---@return Toolbar
function Toolbar.create(parent)
    local toolbar = parent.add { type = "frame", name = "toolbar_" .. Toolbar.nextOrdinal(parent), direction = "vertical", style = "toolbar_frame" }
    toolbar.location = { x = 760, y = 600 }
    Header.create(toolbar)
    Table.create(toolbar):addRow()
    return Toolbar.load(toolbar)
end

---@private
function Toolbar.nextOrdinal(parent)
    return Toolbar.highestOrdinal(parent) + 1
end

---@private
function Toolbar.highestOrdinal(parent)
    local highestOrdinal = 0
    for _, sibling in pairs(parent.children) do
        if Toolbar.is(sibling) then
            local index = Toolbar.load(sibling):ordinal()
            highestOrdinal = index > highestOrdinal and index or highestOrdinal
        end
    end
    return highestOrdinal
end

---@return number
function Toolbar:ordinal()
    return tonumber(self._element.name:match(Toolbar.pattern))
end

---@param element LuaGuiElement
---@return Toolbar
function Toolbar.load(element)
    local this = Toolbar:super()
    this._element = Toolbar.getToolbarElementFromHierarchy(element)
    return this
end

function Toolbar.getToolbarElementFromHierarchy(element)
    if Toolbar.is(element) then
        return element
    else
        return Toolbar.getToolbarElementFromHierarchy(element.parent)
    end
end

---@param guiEvent GuiEvent
function Toolbar:handle(guiEvent)
    local element = guiEvent:element()
    if Slot.is(element) then
        Slot.load(element):handle(guiEvent)
    elseif Header.is(element.parent) then
        Header.load(element):handle(guiEvent)
    end
end

function Toolbar:addRow()
    self:table():addRow()
end

function Toolbar:removeRow()
    self:table():removeLastRow()
end

function Toolbar:addColumn()
    self:table():addColumn()
end

function Toolbar:removeColumn()
    self:table():removeLastColumn()
end

---@public
function Toolbar:update()
    self:table():update()
end

---@return Table
function Toolbar:table()
    return Table.load(self._element.inner_panel.table)
end

function Toolbar:destroy()
    self._element.destroy()
end
