require('lang/Object')
require('model/GuiEvent')
require('model/slot/states/Empty')
require('model/slot/states/Item')
require('model/slot/states/Tool')

---@class Slot : Object
---@field private _state State
Slot = extends(Object)

---@public
---@param element LuaGuiElement
function Slot.is(element)
    return Empty:is(element) or Item:is(element) or Tool:is(element)
end

---@public
---@param parent LuaGuiElement
function Slot.create(parent)
    return Slot.new(Empty.create(parent))
end

---@public
---@param element LuaGuiElement
---@return Slot
function Slot.load(element)
    return Slot.new(Slot.loadState(element))
end

---@private
---@param element LuaGuiElement
---@return State
function Slot.loadState(element)
    if Empty:is(element) then
        return Empty.load(element)
    elseif Item:is(element) then
        return Item.load(element)
    elseif Tool:is(element) then
        return Tool.load(element)
    else
        error("Unknown state")
    end
end

---@private
---@param state State
---@return Slot
function Slot.new(state)
    local this = Slot:super()
    this._state = Object.requireNonNil(state)
    return this
end

---@public
---@param guiEvent GuiEvent
function Slot:handle(guiEvent)
    self._state:handle(guiEvent)
end
