require('lang/Object')
require('model/Cursor')
require('model/GuiEvent')
require('model/Inventory')

---@class State : Object
---@field private __lastPickedSlot State
---@field private _element LuaGuiElement
---@field private _player_index number
State = extends(Object)
State.__lastPickedSlot = nil

---@protected
---@param itemName string
---@return boolean
function State.wasDragged(itemName)
    return State.__lastPickedSlot and State.__lastPickedSlot:isValid() and State.__lastPickedSlot:hasItem()
            and State.__lastPickedSlot:itemName() == itemName
end

---@protected
function State.drag()
    State.__lastPickedSlot:clear()
end

---@protected
---@param self State
function State.rememberPick(self)
    State.__lastPickedSlot = self
end

---@protected
function State.forgetLastPick()
    State.__lastPickedSlot = nil
end

---@public
---@param element LuaGuiElement
---@return boolean
function State:is(element)
    return self.type == State.load(element):getType()
end

---@protected
---@param parent LuaGuiElement
---@param definition LuaGuiElement
function State.create(parent, definition)
    definition.name = definition.name .. "_0"
    local element = parent.add(definition)
    local this = State.load(element)
    this:updateName()
    return this
end

---@protected
---@return State
---@param element LuaGuiElement
function State.load(element)
    local this = State:super()
    this._element = Object.requireNonNil(element)
    this._player_index = this._element.player_index
    return this
end

---@protected
---@return Cursor
function State:cursor()
    return Cursor.load(self._player_index)
end

---@protected
---@return Inventory
function State:inventory()
    return Inventory.load(self._player_index)
end

---@private
---@param replacement State
function State:replaceWith(replacement)
    self:parent().swap_children(self:ordinal(), replacement:ordinal())
    self:destroy()
    replacement:updateName()
end

---@private
function State:destroy()
    self._element.destroy()
end

---@private
function State:updateName()
    self._element.name = self:getType() .. "_" .. self:ordinal()
end

---@private
---@return string
function State:getType()
    return self:name():match("^(%a+)_%d+$")
end

---@private
---@return string
function State:name()
    return self._element.name
end

---@private
---@return number
function State:ordinal()
    return self._element.get_index_in_parent()
end

---@protected
function State:clear()
    self:replaceWith(Empty.create(self:parent()))
end

---@private
---@return LuaGuiElement
function State:parent()
    return self._element.parent
end

---@private
---@return LuaGuiElement
function State:element()
    return self._element
end

---@protected
---@return boolean
function State:hasItem()
    return self:itemName() ~= nil
end

---@private
---@return boolean
function State:isValid()
    return self._element.valid
end

---@protected
---@return string
function State:itemName()
    error("Not implemented")
end

---@public
---@param guiEvent GuiEvent
function State:handle(guiEvent)
    error("Not implemented")
end
