require('lang/Object')
require('model/slot/State')

---@class Empty : State
Empty = extends(State)
Empty.type = "empty"

---@param parent LuaGuiElement
---@return Empty
function Empty.create(parent)
    local definition = {
        name = Empty.type,
        type = "choose-elem-button",
        elem_type = "item",
        style = "toolbar_table_slot",
        mouse_button_filter = { "left", "middle" },
    }
    return Empty:super(State.create(parent, definition))
end

---@param element LuaGuiElement
---@return Empty
function Empty.load(element)
    return Empty:super(State.load(element))
end

function Empty:itemName()
    return self:element().elem_value
end

function Empty:handle(guiEvent)
    local cursor = self:cursor()

    if guiEvent:isLeftClick() then
        if cursor:holdsSomething() then
            if cursor:holdsTool() then
                self:setTool(cursor:itemName())
            elseif cursor:holdsItem() then
                -- some items does not trigger chooser change
                self:setItem(cursor:itemName())
            end
            self:handleItemDrag()
        end
    elseif guiEvent:isElementChange() then
        if cursor:holdsItem() then
            self:setItem(cursor:itemName())
            self:handleItemDrag()
        elseif self:hasItem() then
            self:setItem(self:itemName())
        end
    end
end

function Empty:handleItemDrag()
    local cursor = self:cursor()
    if State.wasDragged(self:cursor():itemName()) then
        State.drag()
    end
    cursor:clear()
    State.forgetLastPick()
end

---@private
---@param itemName string
function Empty:setItem(itemName)
    self:replaceWith(Item.create(self:parent(), itemName))
end

---@private
---@param toolName string
function Empty:setTool(toolName)
    self:replaceWith(Tool.create(self:parent(), toolName))
end
