require('lang/Object')
require('model/slot/State')
require('model/Inventory')

---@class Item : State
Item = extends(State)
Item.type = "item"

---@param parent LuaGuiElement
---@return Item
function Item.create(parent, itemName)
    local definition = {
        name = Item.type,
        type = "sprite-button",
        elem_tooltip = { type = "item", name = itemName },
        sprite = "item/" .. Object.requireNonNil(itemName),
        style = "toolbar_table_slot",
        mouse_button_filter = { "left", "middle" },
    }
    local item = Item:super(State.create(parent, definition))
    item:updateCount()
    return item
end

---@param element LuaGuiElement
---@return Item
function Item.load(element)
    return Item:super(State.load(element))
end

function Item:itemName()
    return self:element().elem_tooltip.name
end

---@param guiEvent GuiEvent
function Item:handle(guiEvent)
    if guiEvent:isLeftClick() then
        local itemName = self:itemName()
        if self:inventory():hasItem(itemName) then
            self:inventory():pick(itemName)
        else
            self:cursor():grabGhost(itemName)
        end
        State.rememberPick(self)
    elseif guiEvent:isMiddleClick() then
        self:clear()
    end
end

function Item:updateCount()
    local count = self:inventory():countOf(self:itemName())
    self._element.number = count > 0 and count or nil
end
